@@ -86,7 +86,6 @@ cb_rstto_nav_file_changed(RsttoNavigator *navigator, gint nr, RsttoNavigatorEntr
 static gboolean window_fullscreen = FALSE;
 static gboolean viewer_scale = 1.0;
 static GtkWidget *menu_bar;
-static GtkWidget *image_tool_bar;
 static GtkWidget *app_tool_bar;
 static GtkWidget *status_bar;
 static gboolean playing = FALSE;
@@ -177,6 +176,7 @@ int main(int argc, char **argv)
                     g_free(path_name);
                     filename = g_dir_read_name(dir);
                 }
+                rstto_navigator_jump_first(navigator);
             }
             gchar *uri = thunar_vfs_path_dup_uri(info->path);
             gtk_recent_manager_add_item(recent_manager, uri);
@@ -192,7 +192,6 @@ int main(int argc, char **argv)
     main_hbox = gtk_hbox_new(0, FALSE);
     main_vbox1 = gtk_vbox_new(0, FALSE);
     menu_bar = gtk_menu_bar_new();
-    image_tool_bar = gtk_toolbar_new();
     app_tool_bar = gtk_toolbar_new();
     status_bar = gtk_statusbar_new();
 
@@ -311,8 +310,6 @@ int main(int argc, char **argv)
 
 
     gtk_container_add(GTK_CONTAINER(s_window), viewer);
-    gtk_toolbar_set_orientation(GTK_TOOLBAR(image_tool_bar), GTK_ORIENTATION_VERTICAL);
-    gtk_box_pack_start(GTK_BOX(main_hbox), image_tool_bar, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(main_hbox), main_vbox1, TRUE, TRUE, 0);
 
     gtk_box_pack_start(GTK_BOX(main_vbox1), s_window, TRUE, TRUE, 0);
@@ -335,13 +332,13 @@ int main(int argc, char **argv)
     gtk_box_pack_start(GTK_BOX(main_vbox), main_hbox, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(main_vbox), status_bar, FALSE, TRUE, 0);
 
-    rstto_picture_viewer_fit_scale(RSTTO_PICTURE_VIEWER(viewer));
+    //rstto_picture_viewer_fit_scale(RSTTO_PICTURE_VIEWER(viewer));
 
-    gtk_toolbar_insert(GTK_TOOLBAR(image_tool_bar), zoom_fit, 0);
-    gtk_toolbar_insert(GTK_TOOLBAR(image_tool_bar), zoom_100, 0);
-    gtk_toolbar_insert(GTK_TOOLBAR(image_tool_bar), zoom_out, 0);
-    gtk_toolbar_insert(GTK_TOOLBAR(image_tool_bar), zoom_in, 0);
-    //gtk_toolbar_insert(GTK_TOOLBAR(image_tool_bar), spacer, 0);
+    gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), zoom_fit, 0);
+    gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), zoom_100, 0);
+    gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), zoom_out, 0);
+    gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), zoom_in, 0);
+    gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), spacer, 0);
     gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), forward, 0);
     gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), previous, 0);
     gtk_toolbar_insert(GTK_TOOLBAR(app_tool_bar), separator, 0);
@@ -513,7 +510,6 @@ cb_rstto_fullscreen(GtkWidget *widget, GdkEventWindowState *event, RsttoPictureV
         {
             window_fullscreen = TRUE;
             gtk_widget_hide(menu_bar);
-            gtk_widget_hide(image_tool_bar);
             gtk_widget_hide(app_tool_bar);
             gtk_widget_hide(status_bar);
             viewer_scale = rstto_picture_viewer_get_scale(viewer);
@@ -523,7 +519,6 @@ cb_rstto_fullscreen(GtkWidget *widget, GdkEventWindowState *event, RsttoPictureV
         {
             window_fullscreen = FALSE;
             gtk_widget_show(menu_bar);
-            gtk_widget_show(image_tool_bar);
             gtk_widget_show(app_tool_bar);
             gtk_widget_show(status_bar);
             if(viewer_scale)
@@ -673,9 +668,11 @@ cb_rstto_key_press_event(GtkWidget *widget, GdkEventKey *event, RsttoNavigator *
                 rstto_navigator_jump_last(navigator);
                 break;
             case GDK_Page_Down:
+            case GDK_space:
                 rstto_navigator_jump_forward(navigator);
                 break;
             case GDK_Page_Up:
+            case GDK_BackSpace:
                 rstto_navigator_jump_back(navigator);
                 break;
         }
@@ -716,10 +713,17 @@ cb_rstto_open(GtkToolItem *item, RsttoNavigator *navigator)
             g_free(file_media);
             thunar_vfs_path_unref(path);
         }
-
-
+        else
+        {
+            gtk_widget_destroy(dialog);
+            dialog = gtk_message_dialog_new(GTK_WINDOW(window),
+                                            GTK_DIALOG_MODAL,
+                                            GTK_MESSAGE_ERROR,
+                                            GTK_BUTTONS_OK,
+                                            _("Could not open file"));
+            gtk_dialog_run(GTK_DIALOG(dialog));
+        }
     }
-
     gtk_widget_destroy(dialog);
 }
 
@@ -763,6 +767,7 @@ cb_rstto_open_dir(GtkToolItem *item, RsttoNavigator *navigator)
                 g_free(path_name);
                 filename = g_dir_read_name(dir);
             }
+            rstto_navigator_jump_first(navigator);
             gchar *uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
             gtk_recent_manager_add_item(recent_manager, uri);
             g_free(uri);
@@ -813,6 +818,7 @@ cb_rstto_open_recent(GtkRecentChooser *chooser, RsttoNavigator *navigator)
                     g_free(path_name);
                     filename = g_dir_read_name(dir);
                 }
+                rstto_navigator_jump_first(navigator);
                 g_free(dir_path);
             }
             gchar *uri = thunar_vfs_path_dup_uri(info->path);
