@@ -27,11 +27,9 @@ struct _RsttoPictureViewerPriv
 {
     GdkPixbuf        *src_pixbuf;
     GdkPixbuf        *dst_pixbuf; /* The pixbuf which ends up on screen */
-    gdouble           scale;
-    gboolean          scale_fts; /* Scale image to fit to screen */
+    RsttoNavigator   *navigator;
     void             (*cb_value_changed)(GtkAdjustment *, RsttoPictureViewer *);
     gboolean          show_border;
-    RsttoNavigator   *navigator;
 };
 
 static void
@@ -47,14 +45,9 @@ static void
 rstto_picture_viewer_size_allocate(GtkWidget *, GtkAllocation *);
 static void
 rstto_picture_viewer_realize(GtkWidget *);
-static void
-rstto_picture_viewer_unrealize(GtkWidget *);
 static gboolean 
 rstto_picture_viewer_expose(GtkWidget *, GdkEventExpose *);
 
-static void
-cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *nav, gint nr, RsttoNavigatorEntry *entry, RsttoPictureViewer *viewer);
-
 static void
 rstto_picture_viewer_paint(GtkWidget *widget);
 static gboolean
@@ -64,7 +57,11 @@ static void
 rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *, GtkAdjustment *, GtkAdjustment *);
 
 static void
-cb_rstto_picture_viewer_value_changed(GtkAdjustment *adjustment, RsttoPictureViewer *viewer);
+cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *, gint , RsttoNavigatorEntry *, RsttoPictureViewer *);
+static void
+cb_rstto_picture_viewer_value_changed(GtkAdjustment *, RsttoPictureViewer *);
+static void
+cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *, GdkEventScroll *);
 
 
 static GtkWidgetClass *parent_class = NULL;
@@ -105,9 +102,9 @@ rstto_picture_viewer_init(RsttoPictureViewer *viewer)
     viewer->priv->dst_pixbuf = NULL;
     gtk_widget_set_redraw_on_allocate(GTK_WIDGET(viewer), TRUE);
 
-    viewer->priv->scale = 1;
-    viewer->priv->scale_fts = FALSE;
     viewer->priv->show_border = TRUE;
+
+    g_signal_connect(G_OBJECT(viewer), "scroll_event", G_CALLBACK(cb_rstto_picture_viewer_scroll_event), NULL);
 }
 
 static void
@@ -124,7 +121,6 @@ rstto_picture_viewer_class_init(RsttoPictureViewerClass *viewer_class)
     viewer_class->set_scroll_adjustments = rstto_picture_viewer_set_scroll_adjustments;
 
     widget_class->realize = rstto_picture_viewer_realize;
-    widget_class->unrealize = rstto_picture_viewer_unrealize;
     widget_class->expose_event = rstto_picture_viewer_expose;
 
     widget_class->size_request = rstto_picture_viewer_size_request;
@@ -203,11 +199,6 @@ rstto_picture_viewer_realize(GtkWidget *widget)
     gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
 }
 
-static void
-rstto_picture_viewer_unrealize(GtkWidget *widget)
-{
-}
-
 static gboolean
 rstto_picture_viewer_expose(GtkWidget *widget, GdkEventExpose *event)
 {
@@ -365,6 +356,8 @@ rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *viewer, GtkAdjus
 static void
 cb_rstto_picture_viewer_value_changed(GtkAdjustment *adjustment, RsttoPictureViewer *viewer)
 {
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+    gdouble scale = rstto_navigator_entry_get_scale(entry);
     if (viewer->priv->src_pixbuf)
     {
         gdouble width = (gdouble)gdk_pixbuf_get_width(viewer->priv->src_pixbuf);
@@ -374,14 +367,14 @@ cb_rstto_picture_viewer_value_changed(GtkAdjustment *adjustment, RsttoPictureVie
         if (viewer->hadjustment && viewer->vadjustment)
         {
             tmp_pixbuf = gdk_pixbuf_new_subpixbuf(viewer->priv->src_pixbuf,
-                            viewer->hadjustment->value / viewer->priv->scale >= 0?
-                                viewer->hadjustment->value / viewer->priv->scale : 0,
-                            viewer->vadjustment->value / viewer->priv->scale >= 0?
-                                viewer->vadjustment->value / viewer->priv->scale : 0,
-                            ((GTK_WIDGET(viewer)->allocation.width/viewer->priv->scale)) < width?
-                                GTK_WIDGET(viewer)->allocation.width/viewer->priv->scale:width,
-                            ((GTK_WIDGET(viewer)->allocation.height/viewer->priv->scale)) < height?
-                                GTK_WIDGET(viewer)->allocation.height/viewer->priv->scale:height);
+                            viewer->hadjustment->value / scale >= 0?
+                                viewer->hadjustment->value / scale : 0,
+                            viewer->vadjustment->value / scale >= 0?
+                                viewer->vadjustment->value / scale : 0,
+                            ((GTK_WIDGET(viewer)->allocation.width/scale)) < width?
+                                GTK_WIDGET(viewer)->allocation.width/scale:width,
+                            ((GTK_WIDGET(viewer)->allocation.height/scale)) < height?
+                                GTK_WIDGET(viewer)->allocation.height/scale:height);
         }
 
         if(viewer->priv->dst_pixbuf)
@@ -392,9 +385,11 @@ cb_rstto_picture_viewer_value_changed(GtkAdjustment *adjustment, RsttoPictureVie
 
         if(tmp_pixbuf)
         {
+            gint dst_width = gdk_pixbuf_get_width(tmp_pixbuf)*scale;
+            gint dst_height = gdk_pixbuf_get_height(tmp_pixbuf)*scale;
             viewer->priv->dst_pixbuf = gdk_pixbuf_scale_simple(tmp_pixbuf,
-                                    gdk_pixbuf_get_width(tmp_pixbuf)*viewer->priv->scale,
-                                    gdk_pixbuf_get_height(tmp_pixbuf)*viewer->priv->scale,
+                                    dst_width>0?dst_width:1,
+                                    dst_height>0?dst_height:1,
                                     GDK_INTERP_BILINEAR);
             g_object_unref(tmp_pixbuf);
             tmp_pixbuf = NULL;
@@ -418,61 +413,104 @@ rstto_picture_viewer_new(RsttoNavigator *navigator)
 void
 rstto_picture_viewer_set_scale(RsttoPictureViewer *viewer, gdouble scale)
 {
-    g_return_if_fail(scale > 0);
-    viewer->priv->scale_fts = FALSE;
-    viewer->priv->scale = scale;
-
-    if(rstto_picture_viewer_refresh(viewer))
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+    if (entry)
     {
-        rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+        rstto_navigator_entry_set_scale(entry, scale);
+        rstto_navigator_entry_set_fit_to_screen (entry, FALSE);
+
+        if(rstto_picture_viewer_refresh(viewer))
+        {
+            rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+        }
     }
 }
 
 gdouble
 rstto_picture_viewer_fit_scale(RsttoPictureViewer *viewer)
 {
-    viewer->priv->scale_fts = TRUE;
-
-    if(rstto_picture_viewer_refresh(viewer))
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+    if (entry)
     {
-        rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+        rstto_navigator_entry_set_fit_to_screen (entry, TRUE);
+
+        if(rstto_picture_viewer_refresh(viewer))
+        {
+            rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+        }
+        return rstto_navigator_entry_get_scale(entry);
     }
-    return viewer->priv->scale;
+    return 0;
 }
 
 gdouble
 rstto_picture_viewer_get_scale(RsttoPictureViewer *viewer)
 {
-    return viewer->priv->scale;
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+    if (entry)
+    {
+        return rstto_navigator_entry_get_scale(entry);
+    }
+    return 0;
 }
 
 static gboolean
 rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
 {
     GtkWidget *widget = GTK_WIDGET(viewer);
-    gboolean changed;
+    gboolean fit_to_screen = FALSE;
+    gdouble scale = 0;
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+    gboolean changed = TRUE;
+    if (entry)
+    {
+        fit_to_screen = rstto_navigator_entry_get_fit_to_screen(entry);
+        scale = rstto_navigator_entry_get_scale(entry);
+    }
+    
+
     gboolean vadjustment_changed = FALSE;
     gboolean hadjustment_changed = FALSE;
     if(viewer->priv->src_pixbuf)
     {
+        gdouble width = (gdouble)gdk_pixbuf_get_width(viewer->priv->src_pixbuf);
+        gdouble height = (gdouble)gdk_pixbuf_get_height(viewer->priv->src_pixbuf);
+        if (scale == 0)
+        {
+            if ((widget->allocation.width > width) && (widget->allocation.height > height))
+            {
+                scale = 1.0;
+                rstto_navigator_entry_set_scale(entry, scale);
+                fit_to_screen = FALSE;
+            }
+            else
+            {
+                fit_to_screen = TRUE;
+                rstto_navigator_entry_set_fit_to_screen(entry, TRUE);
+            }
+        }
 
-        if(viewer->priv->scale_fts)
+        if(fit_to_screen)
         {
-            gdouble width = (gdouble)gdk_pixbuf_get_width(viewer->priv->src_pixbuf);
-            gdouble height = (gdouble)gdk_pixbuf_get_height(viewer->priv->src_pixbuf);
             gdouble h_scale = GTK_WIDGET(viewer)->allocation.width / width;
             gdouble v_scale = GTK_WIDGET(viewer)->allocation.height / height;
             if(h_scale < v_scale)
             {
-                if(viewer->priv->scale != h_scale)
+                if(scale != h_scale)
+                {
+                    scale = h_scale;
                     changed = TRUE;
-                viewer->priv->scale = h_scale;
+                }
+                rstto_navigator_entry_set_scale(entry, h_scale);
             }
             else
             {
-                if(viewer->priv->scale != v_scale)
+                if(scale != v_scale)
+                {
+                    scale = v_scale;
                     changed = TRUE;
-                viewer->priv->scale = v_scale;
+                }
+                rstto_navigator_entry_set_scale(entry, v_scale);
             }
         }
         if(GTK_WIDGET_REALIZED(widget))
@@ -487,7 +525,7 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
             if(viewer->hadjustment)
             {
                 viewer->hadjustment->page_size = widget->allocation.width;
-                viewer->hadjustment->upper = width * viewer->priv->scale;
+                viewer->hadjustment->upper = width * scale;
                 viewer->hadjustment->lower = 0;
                 viewer->hadjustment->step_increment = 1;
                 viewer->hadjustment->page_increment = 100;
@@ -500,7 +538,7 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
             if(viewer->vadjustment)
             {
                 viewer->vadjustment->page_size = widget->allocation.height;
-                viewer->vadjustment->upper = height * viewer->priv->scale;
+                viewer->vadjustment->upper = height * scale;
                 viewer->vadjustment->lower = 0;
                 viewer->vadjustment->step_increment = 1;
                 viewer->vadjustment->page_increment = 100;
@@ -515,15 +553,23 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
             GdkPixbuf *tmp_pixbuf = NULL;
             if (viewer->vadjustment && viewer->hadjustment)
             {
-                tmp_pixbuf = gdk_pixbuf_new_subpixbuf(viewer->priv->src_pixbuf,
-                                                      viewer->hadjustment->value / viewer->priv->scale >= 0?
-                                                        viewer->hadjustment->value / viewer->priv->scale : 0,
-                                                      viewer->vadjustment->value / viewer->priv->scale >= 0?
-                                                        viewer->vadjustment->value / viewer->priv->scale : 0,
-                                                      ((widget->allocation.width/viewer->priv->scale)) < width?
-                                                        widget->allocation.width/viewer->priv->scale:width,
-                                                      ((widget->allocation.height/viewer->priv->scale))< height?
-                                                        widget->allocation.height/viewer->priv->scale:height);
+                if (scale != 1.0)
+                {
+                    tmp_pixbuf = gdk_pixbuf_new_subpixbuf(viewer->priv->src_pixbuf,
+                                                      viewer->hadjustment->value / scale >= 0?
+                                                        viewer->hadjustment->value / scale : 0,
+                                                      viewer->vadjustment->value / scale >= 0?
+                                                        viewer->vadjustment->value / scale : 0,
+                                                      ((widget->allocation.width/scale)) < width?
+                                                        widget->allocation.width/scale:width,
+                                                      ((widget->allocation.height/scale))< height?
+                                                        widget->allocation.height/scale:height);
+                }
+                else
+                {
+                    tmp_pixbuf = viewer->priv->src_pixbuf;
+                    g_object_ref(tmp_pixbuf);
+                }
             }
 
             if(viewer->priv->dst_pixbuf)
@@ -534,10 +580,12 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
 
             if(tmp_pixbuf)
             {
-                viewer->priv->dst_pixbuf = gdk_pixbuf_scale_simple(tmp_pixbuf, 
-                                                     gdk_pixbuf_get_width(tmp_pixbuf) * viewer->priv->scale, 
-                                                     gdk_pixbuf_get_height(tmp_pixbuf) * viewer->priv->scale,
-                                                     GDK_INTERP_BILINEAR);
+                gint dst_width = gdk_pixbuf_get_width(tmp_pixbuf)*scale;
+                gint dst_height = gdk_pixbuf_get_height(tmp_pixbuf)*scale;
+                viewer->priv->dst_pixbuf = gdk_pixbuf_scale_simple(tmp_pixbuf,
+                                        dst_width>0?dst_width:1,
+                                        dst_height>0?dst_height:1,
+                                        GDK_INTERP_BILINEAR);
                 g_object_unref(tmp_pixbuf);
                 tmp_pixbuf = NULL;
             }
@@ -578,8 +626,11 @@ cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *nav, gint nr, RsttoNavi
     if(viewer->priv->src_pixbuf)
     {
         g_object_ref(viewer->priv->src_pixbuf);
-        rstto_picture_viewer_refresh(viewer);
-        rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+        if (GTK_WIDGET_REALIZED(widget))
+        {
+            rstto_picture_viewer_refresh(viewer);
+            rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+        }
     }
 
     if(GTK_WIDGET_REALIZED(widget))
@@ -589,3 +640,28 @@ cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *nav, gint nr, RsttoNavi
         gdk_cursor_unref(cursor);
     }
 }
+
+static void
+cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *viewer, GdkEventScroll *event)
+{
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+    gdouble scale = rstto_navigator_entry_get_scale(entry);
+    switch(event->direction)
+    {
+        case GDK_SCROLL_UP:
+        case GDK_SCROLL_LEFT:
+            rstto_navigator_entry_set_scale(entry, scale / 1.2);
+            rstto_navigator_entry_set_fit_to_screen (entry, FALSE);
+            break;
+        case GDK_SCROLL_DOWN:
+        case GDK_SCROLL_RIGHT:
+            rstto_navigator_entry_set_scale(entry, scale * 1.2);
+            rstto_navigator_entry_set_fit_to_screen (entry, FALSE);
+            break;
+    }
+    if(rstto_picture_viewer_refresh(viewer))
+    {
+        rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+    }
+
+}
